open Term

(* single simple step *)
type step    = t -> t list
(* full step *)
type rewrite = t * (Rule.t * position) option * t
type stepS   = t -> rewrite list list

val rewrite_a   : Trs.t -> t -> t
val rewrite     : Trs.t -> step
val multi_step  : Trs.t -> step
(* special *)
val stepS       : Trs.t -> stepS
val multi_stepS : Trs.t -> stepS
(* limited by k *)
val k_stepS     : int -> Trs.t -> stepS
val k_step      : int -> Trs.t -> step
(* and including zero step (i.e. '=')*)
val k_stepSs    : int -> Trs.t -> stepS
val k_steps     : int -> Trs.t -> step

(* elimination *)
val reduce     : int -> Trs.t -> Trs.t
val reduce_by  : int -> (Trs.t -> step) -> Trs.t -> Trs.t
val if_reduced : Trs.t -> Trs.t -> unit

(* predicate *)
val is_NF       : Trs.t -> t -> bool

(* need terminating TRS *)
val normalform  : Trs.t -> t -> t
val normalforms : Trs.t -> t -> t list

(* utilities *)
val last         : rewrite list -> rewrite
val last_term    : rewrite list -> t
val nontrivials  : rewrite list -> rewrite list
val shrink       : stepS -> step
val show_rewrite : rewrite -> unit
val show_stepS   : rewrite list list -> unit
val cache_on     : unit -> unit
val cache_off    : unit -> unit
